function PadeShamPoly2(fx,gx,xRange,Lb,Ub,runNum,sFxName,diaryFilename)
%PADESHAMPOLY2 Summary of this function goes here
%   Detailed explanation goes here
clc
global bDeleteIfExists
global bUseDiary
global xdata
global ydata
global orderA
global orderB 
global ggx

warning('off','all')
if isempty(sFxName)
  sFxName = getFuncName(fx);
end
xdata = xRange';
ydata = xdata;
for i=1:length(xdata)
  ydata(i)=fx(xdata(i));
end
ggx = gx;

xmin = min(xdata);
xmax = max(xdata);
ymin = min(ydata);
ymax = max(ydata);
xdata = (xdata - xmin)/(xmax - xmin) + 1;
ydata = (ydata - ymin)/(ymax - ymin) + 1;
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);

options = optimoptions('particleswarm', 'Display', 'iter');
[x,psAICc] = particleswarm(@optimFunc,4,Lb,Ub,options);
A = x(1);
B = x(2);
orderA = round(x(3));
orderB = round(x(4));
if bUseDiary
  diaryFilename = strrep(diaryFilename, ".txt", strcat("_", num2str(orderA),"_", num2str(orderB),"_run", num2str(runNum),".txt"));
  if exist(diaryFilename, 'file')==2
    if bDeleteIfExists
      delete(diaryFilename);
    else
      return;
    end
  end
end
X = [];
for i=1:orderA
  xs = xdata.^gx(i,A,B);
  X = [X;xs']; 
end
for i=1:orderB
  xs = ydata.*(xdata.^gx(i,A,B));
  X = [X;xs']; 
end
X = X';
lm = fitlm(X,ydata);
if bUseDiary
  diary(diaryFilename)
end
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);
sp = getFuncName(gx);
fprintf('Pade-Shammas polynomial power is %s\n', sp);
format long
disp(lm);
anva = anova(lm,'summary');
disp(anva);
format short
fprintf('A = %f, B = %f\n', A, B);
fprintf('order Q(x) = %f, order D(x) = %f\n', orderA, orderB);
fprintf("Xmin = %f and Xmax = %f\n", xmin, xmax);
fprintf("Ymin = %f and Ymax = %f\n", ymin, ymax);
lstPwrA = zeros(orderA,1);
for i=1:orderA
  lstPwrA(i) = gx(i,A,B);
end
lstPwrB = zeros(orderB,1);
for i=1:orderB
  lstPwrB(i) = gx(i,A,B);
end
fprintf('List of powers for Q(x): [');
for i=1:orderA-1
 fprintf('%f, ', lstPwrA(i));
end
fprintf('%f]\n', lstPwrA(orderA));
fprintf('List of powers for D(x): [');
for i=1:orderB-1
 fprintf('%f, ', lstPwrB(i));
end
fprintf('%f]\n', lstPwrB(orderB));
fprintf('Fitting %s in range (%f, %f)\n', sFxName, xmin ,xmax);
n = length(xdata);
sumsqr = 0;
for i=1:n
  qx = lm.Coefficients{1,1};
  for j=2:orderA+1
    qx = qx + lm.Coefficients{j,1} * xdata(i)^lstPwrA(j-1);
  end
  dx = 1;
  k = 1;
  for j=orderA+2:orderA+1+orderB
    dx = dx - lm.Coefficients{j,1} * xdata(i)^lstPwrB(k);
    k = k + 1;
  end
  yc = qx/dx;
  sumsqr = sumsqr + (ydata(i) - yc)^2;
end  
k = orderA + orderB + 1;
fprintf('MSS of errors squared = %e\n', sqrt(sumsqr)/n);
fprintf('Corrected MSS of errors squared = %e\n', sqrt(sumsqr)/n*sqrt(2));
fprintf("R-Squared = %12.8f\n", lm.Rsquared.Ordinary);
fprintf("R-Squared Adjusted = %12.8f\n", lm.Rsquared.Adjusted);
AIC = lm.ModelCriterion.AIC;
AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
fprintf('Particle swarm AICc =  %e\n', psAICc);
fprintf('AIC =  %e\n', AIC);
fprintf('AICc =  %e\n', AICc);

if bUseDiary
  diary off
end
end

function AICc = optimFunc(x)
  global xdata
  global ydata
  global orderA
  global orderB
  global ggx
  
  A = x(1);
  B = x(2);
  orderA = round(x(3));
  orderB = round(x(4));
  X = [];
  for i=1:orderA
    xs = xdata.^ggx(i,A,B);
    X = [X;xs'];
  end
  for i=1:orderB
    xs = ydata.*(xdata.^ggx(i,A,B));
    X = [X;xs'];
  end  
  X = X';
  lm = fitlm(X,ydata);

  n = length(xdata);
  k = orderA + orderB + 1;
  AIC = lm.ModelCriterion.AIC;
  AICc = AIC + 2*k*(1 + (k+1)/(n-k-1));
  if isinf(AICc), AICc = -1e+99; end
end

function sFx = getFuncName(fx)
  sFx = func2str(fx);
  if sFx(1:2)=="@("
    i = strfind(sFx,")");
    sFx = sFx(i(1)+1:end);
  elseif sFx(1)=="@"
    sFx = strcat(sFx(2:end),".m");
  else
    % return sFx as is
  end
end